@@ -90,6 +90,10 @@ impl<'a> Sorter<'a> {
     where
         T: RadixKey + Sized + Send + Copy + Sync,
     {
+        if bucket.len() < 2 {
+            return;
+        }
+
         let plateaus = detect_plateaus(bucket, level);
         let (mut prefix_sums, end_offsets) = apply_plateaus(bucket, counts, &plateaus);
 
@@ -117,7 +121,7 @@ mod tests {
         let sorter = Sorter::new(true, &StandardTuner);
 
         sort_comparison_suite(shift, |inputs| {
-            let counts = get_counts(inputs, T::LEVELS - 1);
+            let (counts, _) = get_counts(inputs, T::LEVELS - 1);
 
             sorter.ska_sort_adapter(inputs, &counts, T::LEVELS - 1);
         });
