@@ -8,8 +8,8 @@ pub struct TuningParams {
 }
 
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
+#[cfg(feature = "multi-threaded")]
 pub enum Algorithm {
-    #[allow(unused)]
     MtOop,
     MtLsb,
     Scanning,
@@ -21,6 +21,15 @@ pub enum Algorithm {
     Ska,
 }
 
+#[derive(Debug, Clone, Copy, Eq, PartialEq)]
+#[cfg(not(feature = "multi-threaded"))]
+pub enum Algorithm {
+    Comparative,
+    LrLsb,
+    Lsb,
+    Ska,
+}
+
 pub trait Tuner {
     fn pick_algorithm(&self, p: &TuningParams, counts: &[usize]) -> Algorithm;
 }
